[libcxx-commits] [PATCH] D99373: [SystemZ][z/OS] correct rc and errno within nanosleep()
Zibi Sarbino via Phabricator via libcxx-commits
libcxx-commits at lists.llvm.org
Tue Apr 6 08:37:20 PDT 2021
This revision was not accepted when it landed; it landed in state "Needs Review".
This revision was automatically updated to reflect the committed changes.
zibi marked an inline comment as done.
Closed by commit rG916093f49faf: [SystemZ][z/OS] correct rc and errno within nanosleep() (authored by zibi).
Changed prior to commit:
https://reviews.llvm.org/D99373?vs=334759&id=335526#toc
Repository:
rG LLVM Github Monorepo
CHANGES SINCE LAST ACTION
https://reviews.llvm.org/D99373/new/
https://reviews.llvm.org/D99373
Files:
libcxx/include/__support/ibm/nanosleep.h
Index: libcxx/include/__support/ibm/nanosleep.h
===================================================================
--- libcxx/include/__support/ibm/nanosleep.h
+++ libcxx/include/__support/ibm/nanosleep.h
@@ -12,27 +12,45 @@
#include <unistd.h>
-inline int nanosleep(const struct timespec* req, struct timespec* rem)
-{
- // The nanosleep() function is not available on z/OS. Therefore, we will call
- // sleep() to sleep for whole seconds and usleep() to sleep for any remaining
- // fraction of a second. Any remaining nanoseconds will round up to the next
- // microsecond.
-
- useconds_t __micro_sec = (rem->tv_nsec + 999) / 1000;
- if (__micro_sec > 999999)
- {
- ++rem->tv_sec;
- __micro_sec -= 1000000;
- }
- while (rem->tv_sec)
- rem->tv_sec = sleep(rem->tv_sec);
- if (__micro_sec) {
- rem->tv_nsec = __micro_sec * 1000;
- return usleep(__micro_sec);
- }
- rem->tv_nsec = 0;
- return 0;
+inline int nanosleep(const struct timespec* __req, struct timespec* __rem) {
+ // The nanosleep() function is not available on z/OS. Therefore, we will call
+ // sleep() to sleep for whole seconds and usleep() to sleep for any remaining
+ // fraction of a second. Any remaining nanoseconds will round up to the next
+ // microsecond.
+ if (__req->tv_sec < 0 || __req->tv_nsec < 0 || __req->tv_nsec > 999999999) {
+ errno = EINVAL;
+ return -1;
+ }
+ useconds_t __micro_sec =
+ static_cast<useconds_t>((__req->tv_nsec + 999) / 1000);
+ time_t __sec = __req->tv_sec;
+ if (__micro_sec > 999999) {
+ ++__sec;
+ __micro_sec -= 1000000;
+ }
+ __sec = sleep(static_cast<unsigned int>(__sec));
+ if (__sec) {
+ if (__rem) {
+ // Updating the remaining time to sleep in case of unsuccessful call to sleep().
+ __rem->tv_sec = __sec;
+ __rem->tv_nsec = __micro_sec * 1000;
+ }
+ errno = EINTR;
+ return -1;
+ }
+ if (__micro_sec) {
+ int __rt = usleep(__micro_sec);
+ if (__rt != 0 && __rem) {
+ // The usleep() does not provide the amount of remaining time upon its failure,
+ // so the time slept will be ignored.
+ __rem->tv_sec = 0;
+ __rem->tv_nsec = __micro_sec * 1000;
+ // The errno is already set.
+ return -1;
+ }
+ return __rt;
+ }
+ return 0;
}
#endif // _LIBCPP_SUPPORT_IBM_NANOSLEEP_H
-------------- next part --------------
A non-text attachment was scrubbed...
Name: D99373.335526.patch
Type: text/x-patch
Size: 2346 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/libcxx-commits/attachments/20210406/4761dbf2/attachment.bin>
More information about the libcxx-commits
mailing list